#| window-snooper -- display as much information as possible about window

   Copyright (C) 2001 Daniel Pfeiffer <occitan@esperanto.org>

   This file is part of sawfish.

   sawfish is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sawfish is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sawfish; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

|#

(define-structure sawfish.wm.ext.window-snooper

    (export window-snooper)

    (open rep
	  rep.io.files
	  rep.io.processes
	  rep.regexp
	  sawfish.wm)


  (defvar window-snooper-functions
    '(desktop-window-p
      get-window-wm-protocols
      transient-children
      transient-group
      window-absolute-position
      window-actual-group-id
      window-avoided-p
      window-border-width
      window-class
      window-dimensions
      window-frame
      window-frame-dimensions
      window-framed-p
      window-frame-offset
      window-full-name
      window-gravity
      window-group-id
      window-iconifiable-p
      window-icon-image
      window-icon-name
      window-id
      window-ignored-p
      window-in-cycle-p
      window-mapped-p
      window-maximizable-p
      window-maximized-horizontally-p
      window-maximized-p
      window-maximized-vertically-p
      window-name
      window-on-top-p
      window-outside-viewport-p
      window-outside-workspace-p
      window-position
      window-really-wants-input-p
      window-shaded-p
      window-shaped-p
      windows-in-group
      window-size-hints
      window-sticky-p
      window-sticky-p/viewport
      window-sticky-p/workspace
      window-transient-p		;
      window-type
      window-unmaximized-dimensions
      window-unmaximized-position
      window-urgent-p
      window-viewport
      window-visibility
      window-visible-p
      window-wants-input-p
      window-workspaces)
    "Functions to display in window-snooper's first tab.
Each of these will be passed the window-object as their only argument.
See also `window-snooper-requires'.")


  (defvar window-snooper-root-functions
    '(current-head
      current-head-dimensions
      current-head-offset
      head-count
      root-window-id
      screen-dimensions
      screen-height
      screen-viewport
      screen-width)
    "Functions to display in window-snooper's first tab for the root-window.
See also `window-snooper-requires'.")


  (defvar window-snooper-requires
    '(sawfish.wm.state.maximize
      sawfish.wm.util.groups)
    "These need to be required for `window-snooper-functions' and/or
`window-snooper-root-functions'.")


  (defvar window-snooper-program
    (expand-file-name "window-snooper" sawfish-exec-directory)
    "Location of the program implementing sawfish's window-snooper.")



  (define (window-snooper w)
    "Display all that sawfish knows about WINDOW.
WINDOW may be the symbol `root', a window object or a numeric window id."
    (unless (windowp w)
      (setq w (or (if (eq w 'root) (root-window-id))
		  (get-window-by-id w)
		  w)))

    (let* ((proc (make-process standard-error))
	   (root (eq w (root-window-id))))

      (start-process proc window-snooper-program
		     (number->string (if (windowp w) (window-id w) w)))
      (mapc require window-snooper-requires)

      (write proc
	(string-replace "#<(.+?)>" "[\254\\1\255]"
	  (format () "%S(%s%s%s%s)\n"
		  (if (windowp w) (window-name w)
		    (if root "root"
		      (or (caddr (get-x-property w 'WM_NAME))
			  "(unnamed)")))

		  (when (or (windowp w) root)
		    `(0
		      ,@(if (windowp w)
			    (mapcar
			     (lambda (f) (list f (eval `(,f ,w))))
			     window-snooper-functions)
			  (mapcar
			   (lambda (f) (list f (eval `(,f))))
			   window-snooper-root-functions))
		      ,@(when root
			  (let loop ((i (1- (head-count)))
				     l)
			       (if (< i 0)
				   l
				 (loop (1- i)
				       `(((head-dimensions ,i)
					  ,(head-dimensions i))
					 ((head-offset ,i)
					  ,(head-offset i))
					 ,@l)))))))

		  (when (windowp w)
		    `(1
		      ,@(let (l)
			  (map-window-properties
			   (lambda (p v) (setq l `((,p ,v) ,@l)))
			   w)
			  l)))

		  (if (windowp w)
		      `(2
			,@(mapcar
			   (lambda (k)
			     (list (if (eq (car (rassoc (cdr k) window-keymap))
					   (car k))
				       'x '-)
				   (intern (event-name (cdr k)))
				   (car k)))
			   (cdr (window-get w 'keymap))))
		    (when root
		      `(3
			,@(mapcar
			   (lambda (k)
			     (list (intern (event-name (cdr k)))
				   (car k)))
			   (cdr root-window-keymap)))))

		  `(4
		    ,@(mapcar
		       (lambda (p) (cons p (get-x-property w p)))
		       (list-x-properties w))))))))

  ;;###autoload
  (define-command 'window-snooper window-snooper #:spec "%W"))
